import typing
import xml.parsers.expat as xml_expat
import xmltodict
import zipfile

import openepub as oe
from openepub.common import aslist


class EpubPackage:
    def __init__(self, epub: "oe.Epub", path: zipfile.Path):
        self.epub = epub
        self.path = path
        self._package = self._read_package()
        if not self._package:
            raise oe.MalformedPackage(f"EPUB package file at {path.at} is malformed.")
        self.items = self._read_manifest()

    def _read_package(self) -> dict:
        try:
            package = xmltodict.parse(self.path.read_text())
            return package["package"]
        except (xml_expat.ExpatError, FileNotFoundError, KeyError, TypeError):
            pass
        return {}

    def _read_manifest(self) -> list:
        items = []
        try:
            for itemref in aslist(self._package["manifest"]["item"]):
                # TODO: In theory, the path can specify a scheme
                # see https://www.w3.org/TR/epub-33/#sec-item-elem
                # assuming it is a relative path for now.
                path = self.path.parent.joinpath(itemref["@href"])
                if not path.exists():
                    continue

                # TODO: Support other media types.
                if itemref["@media-type"] not in ("application/xhtml+xml", "text/html"):
                    continue

                items.append(
                    oe.EpubXHTMLItem(package=self, path=path, id=itemref["@id"])
                )
        except (KeyError, TypeError):
            return []
        return items

    def itermanifest(self):
        raise NotImplementedError("Iterating over the manifest is not yet supported.")

    def iterspine(self) -> typing.Iterator["oe.EpubItem"]:
        try:
            spine = aslist(self._package["spine"]["itemref"])
        except (KeyError, TypeError):
            spine = []
        for itemref in spine:
            if not isinstance(itemref, dict) or "@idref" not in itemref:
                continue
            item = self.get_item(itemref["@idref"])
            if item is None:
                continue
            yield item

    def get_item(self, id: str) -> typing.Union["oe.EpubItem", None]:
        return next(filter(lambda i: i.id == id, self.items), None)



__version__ = "0.0.5"


from .epub.epub import Epub
from .epub.item import EpubItem, EpubXHTMLItem
from .epub.package import EpubPackage
from .exceptions import EpubError, InvalidFileError, MalformedPackage, DRMProtected


__all__ = [
    # Library Version
    __version__,
    # Core Classes
    Epub,
    EpubItem,
    EpubXHTMLItem,
    EpubPackage,
    # Exceptions
    EpubError,
    InvalidFileError,
    MalformedPackage,
]



from tempfile import TemporaryFile
from unittest import TestCase

from openepub import Epub, InvalidFileError


class EpubTestCase(TestCase):
    def test_open_get_epub_obj_valid_path(self):
        epub_1 = Epub("test/mock/tödliche_lektion.epub")
        self.assertIsInstance(epub_1, Epub)
        epub_2 = Epub("test/mock/tödliche_lektion.abcd")
        self.assertIsInstance(epub_2, Epub)

    def test_open_get_epub_obj_invalid_path(self):
        with self.assertRaises(InvalidFileError):
            Epub("test/mock/some_invalid_path.epub")
        with self.assertRaises(InvalidFileError):
            Epub(b"3.14159")

    def test_open_get_epub_obj_silly_path(self):
        with self.assertRaises(TypeError):
            Epub(3.14159)

    def test_open_get_epub_obj_invalid_file(self):
        with self.assertRaises(InvalidFileError):
            Epub("test/mock/not_an_epub.epub")

    def test_epub_get_text(self):
        epub_1 = Epub("test/mock/tödliche_lektion.epub")
        text = epub_1.get_text()
        stripped = text.strip()
        self.assertEqual("Naomi Novik", stripped[:11])
        self.assertEqual("DATENSCHUTZHINWEIS", stripped[-18:])

    def test_epub_get_text_2(self):
        epub_1 = Epub("test/mock/philosophers_stone.epub")
        text = epub_1.get_text()
        stripped = text.strip()
        self.assertTrue(stripped.startswith("Harry Potter and the Philosopher's Stone"))
        self.assertTrue(stripped.endswith("fun with Dudley this summer …’"))

    def test_open_epub_from_stream(self):
        stream = TemporaryFile()
        with open("test/mock/tödliche_lektion.epub", "rb") as f:
            stream.write(f.read())
        epub_1 = Epub(stream=stream)
        self.assertIsInstance(epub_1, Epub)



import bs4
import typing
import xml.parsers.expat as xml_expat
import xmltodict
import zipfile

import openepub as oe


class EpubItem:
    def __init__(self, package: "oe.EpubPackage", path: zipfile.Path, id: str):
        self.package = package
        self.path = path
        self.id = id


class EpubXHTMLItem(EpubItem):
    def __init__(self, package: "oe.EpubPackage", path: zipfile.Path, id: str):
        super().__init__(package, path, id)
        try:
            self.content = self.path.read_text()
        except UnicodeDecodeError:
            raise oe.DRMProtected("This content is protected by DRM.") from None
        self.soup = bs4.BeautifulSoup(self.content, "html.parser")

    def get_text(self):
        for br in self.soup.find_all("br"):
            br.replace_with("\n")
        for tag in self.soup.find_all(["h1", "h2", "h3", "h4", "h5", "h6", "p"]):
            if tag.previous_sibling != "\n":
                tag.insert_before("\n")
        if self.soup.find("head") and self.soup.find("body"):
            return self.soup.find("body").get_text()
        return self.soup.get_text()
